import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Popover, Modal } from 'antd'
import { EditOutlined, ToolOutlined, DeleteOutlined, FontColorsOutlined, UngroupOutlined } from '@ant-design/icons'

import MKEmitter from '@/utils/events.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncIconComponent from '@/utils/asyncIconComponent'
import { resetStyle } from '@/utils/utils-custom.js'
import getSettingForm from './options'
import './index.scss'

const { confirm } = Modal
const NormalForm = asyncIconComponent(() => import('@/components/normalform'))
const CopyComponent = asyncIconComponent(() => import('@/menu/components/share/copycomponent'))
const NormalHeader = asyncComponent(() => import('@/menu/components/share/normalheader'))
const PasteComponent = asyncIconComponent(() => import('@/menu/components/tabs/paste'))
const GroupComponents = asyncComponent(() => import('../groupcomponents'))

class NormalGroup extends Component {
  static propTpyes = {
    group: PropTypes.object,
    deletecomponent: PropTypes.func,
    unGroup: PropTypes.func,
    updateConfig: PropTypes.func,
  }

  state = {
    group: null,
    editab: null,
  }

  UNSAFE_componentWillMount () {
    const { group } = this.props

    if (group.isNew) {
      let _group = {
        uuid: group.uuid,
        type: group.type,
        subtype: group.subtype,
        width: 24,
        name: group.name,
        setting: { width: 24, name: group.name },
        style: { marginLeft: '8px', marginRight: '8px', marginTop: '8px', marginBottom: '8px' },
        components: []
      }
      this.setState({
        group: _group
      })
      this.props.updateConfig(_group)
    } else {
      this.setState({
        group: fromJS(group).toJS()
      })
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
  }

  changeStyle = () => {
    const { group } = this.state

    MKEmitter.emit('changeStyle', ['background', 'border', 'padding', 'margin', 'shadow', 'clear', 'minHeight'], group.style, this.getStyle)
  }

  getStyle = (style) => {
    let _card = {...this.state.group, style}

    this.setState({
      group: _card
    })
    
    this.props.updateConfig(_card)
  }

  updateComponent = (component) => {
    const { group } = this.state

    if (!is(fromJS(group.setting), fromJS(component.setting)) || !is(fromJS(group.style), fromJS(component.style))) {
      // 注册事件-标签变化，通知组内元素
      let ids = []
      group.components.forEach(item => {
        ids.push(item.uuid)
      })
      MKEmitter.emit('tabsChange', ids.join(','))
    }

    component.width = component.setting.width
    component.name = component.setting.name

    this.setState({
      group: component
    })
    this.props.updateConfig(component)
  }

  insert = (item) => {
    let group = fromJS(this.state.group).toJS()

    group.components.push(item)

    this.setState({group})
    this.props.updateConfig(group)
  }

  getWrapForms = () => {
    const { setting } = this.state.group

    return getSettingForm(setting)
  }

  updateWrap = (res) => {
    let group = {...this.state.group, setting: res}

    if (res.title && !group.headerStyle) {
      group.headerStyle = { fontSize: '16px', borderBottomWidth: '1px', borderBottomColor: '#e8e8e8' }
    }

    this.updateComponent(group)
  }

  unGroup = () => {
    const { group } = this.state

    if (group.components.length === 0) return

    const _this = this
    confirm({
      title: '确定释放分组元素吗？',
      content: '',
      onOk() {
        _this.props.unGroup(group.uuid)

        setTimeout(() => {
          _this.updataGroup()
        }, 10)
      },
      onCancel() {}
    })
  }

  updataGroup = () => {
    const { group } = this.props

    if (group.components.length === 0) {
      this.setState({
        group: fromJS(group).toJS()
      })
    }
  }

  render() {
    const { group } = this.state
    let _style = resetStyle(group.style)

    let paddingTop = true
    if (group.style.paddingTop && parseInt(group.style.paddingTop) >= 28) {
      paddingTop = false
    }

    return (
      <div className={'menu-group-edit-box ' + (paddingTop ? 'padding ' : '') + (group.setting.layout || '')} style={_style} id={group.uuid}>
        <NormalHeader hideSearch="true" config={group} updateComponent={this.updateComponent}/>
        <Popover overlayClassName="mk-popover-control-wrap" mouseLeaveDelay={0.2} mouseEnterDelay={0.2} content={
          <div className="mk-popover-control">
            <NormalForm title="分组设置" width={700} update={this.updateWrap} getForms={this.getWrapForms}>
              <EditOutlined style={{color: '#1890ff'}} title="编辑"/>
            </NormalForm>
            <CopyComponent type="group" card={group}/>
            <UngroupOutlined title="释放" style={group.components.length > 0 ? {color: '#32c5d2'} : {color: '#eeeeee', cursor: 'not-allowed'}} onClick={this.unGroup}/>
            <PasteComponent type="group" insert={this.insert} />
            <FontColorsOutlined className="style" title="调整样式" onClick={this.changeStyle}/>
            <DeleteOutlined className="close" title="delete" onClick={() => this.props.deletecomponent(group.uuid)} />
          </div>
        } trigger="hover">
          <ToolOutlined />
        </Popover>
        <GroupComponents config={group} handleList={this.updateComponent} deleteCard={this.deleteCard} />
      </div>
    )
  }
}

export default NormalGroup